(* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *  Copyright (C) 2004, 2005 Milenko "DCoder" Mitrovic                       *
 *  Mail: dcoder@dsp-worx.de                                                 *
 *  Web:  http://www.dsp-worx.de                                             *
 *                                                                           *
 *  This Program is free software; you can redistribute it and/or modify     *
 *  it under the terms of the GNU General Public License as published by     *
 *  the Free Software Foundation; either version 2, or (at your option)      *
 *  any later version.                                                       *
 *                                                                           *
 *  This Program is distributed in the hope that it will be useful,          *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             *
 *  GNU General Public License for more details.                             *
 *                                                                           *
 *  You should have received a copy of the GNU General Public License        *
 *  along with GNU Make; see the file COPYING.  If not, write to             *
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.    *
 *  http://www.gnu.org/copyleft/gpl.html                                     *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *)

unit PropertyPages;

interface

uses
  Windows, DirectShow9, BaseClass, Classes, DirectSound, Messages, CommCtrl,
  SysUtils;

const
  CLSID_PropPageSettings : TGuid = '{3E869E38-0697-4349-8728-29640452B49E}';
  CLSID_PropAbout        : TGuid = '{AC473FFB-364A-4E50-98D4-EA65F3046A84}';

  IID_ICrossfadingRenderer: TGuid = '{86AADA26-8B99-48EF-8F4D-ABD61B978245}';

type
  ICrossfadingRenderer = interface(IUnknown)
  ['{86AADA26-8B99-48EF-8F4D-ABD61B978245}']
    function GetDevice(out Guid: TGuid): HRESULT; stdcall;
    function SetDevice(Guid: TGuid): HRESULT; stdcall;
    function GetFadingSeekStop(out Fading: Integer): HRESULT; stdcall;
    function SetFadingSeekStop(Fading: Integer): HRESULT; stdcall;
    function GetFadingCross(out Fading: Integer): HRESULT; stdcall;
    function SetFadingCross(Fading: Integer): HRESULT; stdcall;
    function GetAllowVideo(out Allow: Boolean): HRESULT; stdcall;
    function SetAllowVideo(Allow: Boolean): HRESULT; stdcall;
  end;

  TPropPageSettings = class(TBCBasePropertyPage)
  private
    FDeviceList: TList;
    FIntf: ICrossfadingRenderer;

    FComboDriver: THandle;
    FEditSeek: THandle;
    FEditStart: THandle;
    FCheckAllowVideo: THandle;

    procedure RefreshUI;
    procedure ChangeEdit(Edit: THandle; Value: Integer);
    procedure ChangeText(Edit: THandle);
  public
    constructor CreateFromFactory(Factory: TBCClassFactory; const Controller: IInterface); override;
    constructor Create(Name: WideString; pUnk: IUnknown; DialogID: Integer; Title: WideString);
    destructor Destroy; override;

    function OnConnect(Unknown: IInterface): HRESULT; override;
    function OnDisconnect: HRESULT; override;
    function OnReceiveMessage(hwndDlg: Cardinal; uMsg: Cardinal; wParam: Integer; lParam: Integer): Integer; override;
    function OnApplyChanges: HRESULT; override;
  end;

  TPropPageAbout = class(TBCBasePropertyPage)
  public
    constructor CreateFromFactory(Factory: TBCClassFactory; const Controller: IInterface); override;
  end;

implementation

const
  IDC_COMBOBOX_DRIVER = 1000;

  IDC_EDIT_FADING_SEEK = 1001;
  IDC_SPIN_FADING_SEEK = 1002;

  IDC_EDIT_FADING_START = 1003;
  IDC_SPIN_FADING_START = 1004;

  IDC_CHECK_ALLOW_VIDEO = 1005;

  MAX_FADING = 10000;

type
  TDSDevice = record
    Name: WideString;
    Guid: TGuid;
  end;
  PDSDevice = ^TDSDevice;

function DSEnumCallback(Guid : PGUID; lpstrDescription : PChar; lpstrModule : PChar; lpContext : Pointer) : BOOL; stdcall;
var
  dev: PDSDevice;
begin
  new (dev);
  dev.Name := lpstrDescription;
  if Assigned(GUID)
    then dev.Guid := Guid^
    else dev.Guid := GUID_NULL;
  TList(lpContext).Add(dev);
  Result := True;
end;

constructor TPropPageSettings.CreateFromFactory(Factory: TBCClassFactory; const Controller: IInterface);
begin
  Create(Factory.Name, Controller, 101, 'Settings');
end;

constructor TPropPageSettings.Create(Name: WideString; pUnk: IUnknown; DialogID: Integer; Title: WideString);
begin
  inherited Create(Name, pUnk, DialogID, Title);
  FDeviceList := TList.Create;
  DirectSoundEnumerate(DSEnumCallback, FDeviceList);
end;

destructor TPropPageSettings.Destroy;
begin
  while(FDeviceList.Count > 0) do
  begin
    Dispose(PDSDevice(FDeviceList[0]));
    FDeviceList.Delete(0);
  end;
  FDeviceList.Free;
  inherited Destroy;
end;

function TPropPageSettings.OnConnect(Unknown: IInterface): HRESULT;
begin
  Unknown.QueryInterface(IID_ICrossfadingRenderer, FIntf);
  Result := S_OK;
end;

function TPropPageSettings.OnDisconnect: HRESULT;
begin
  FIntf := nil;
  Result := S_OK;
end;

function TPropPageSettings.OnReceiveMessage(hwndDlg: Cardinal; uMsg: Cardinal; wParam: Integer; lParam: Integer): Integer;
begin
  case uMsg of
    WM_INITDIALOG:
    begin
      FComboDriver := GetDlgItem(hwndDlg, IDC_COMBOBOX_DRIVER);
      FEditSeek := GetDlgItem(hwndDlg, IDC_EDIT_FADING_SEEK);
      FEditStart := GetDlgItem(hwndDlg, IDC_EDIT_FADING_START);
      FCheckAllowVideo := GetDlgItem(hwndDlg, IDC_CHECK_ALLOW_VIDEO);
      
      SendMessage(FEditSeek, EM_SETLIMITTEXT, 5, 0);
      SendMessage(FEditStart, EM_SETLIMITTEXT, 5, 0);

      RefreshUI;

      Result := LRESULT(1);
      Exit;
    end;
    WM_NOTIFY:
    begin
      case LOWORD(wParam) of
        IDC_SPIN_FADING_SEEK:  ChangeEdit(FEditSeek, PNMUPDOWN(lParam)^.iDelta);
        IDC_SPIN_FADING_START: ChangeEdit(FEditStart, PNMUPDOWN(lParam)^.iDelta);
      end;
    end;
    WM_COMMAND:
    begin
      case LOWORD(wParam) of
        IDC_CHECK_ALLOW_VIDEO: SetPageDirty;
        IDC_COMBOBOX_DRIVER:
        begin
          if HIWORD(wParam) = CBN_SELENDOK
            then SetPageDirty;
        end;
        IDC_EDIT_FADING_SEEK,
        IDC_EDIT_FADING_START:
        begin
          if HIWORD(wParam) = EN_CHANGE
            then ChangeText(lParam);
        end;
      end;
    end;
  end;

  Result := inherited OnReceiveMessage(hwndDlg, uMsg, wParam, lParam);
end;

procedure TPropPageSettings.ChangeText(Edit: THandle);
var
  val: Integer;
  buf: array[0..255] of Char;
begin
  if GetWindowText(Edit, buf, 256) = 0 then
  begin
    SetWindowText(Edit, PChar(inttostr(0)));
    Exit;
  end;

  try
    val := strtoint(buf);
  except
    SetWindowText(Edit, PChar(inttostr(0)));
    SetPageDirty;
    Exit;
  end;

  if val < 0
    then SetWindowText(Edit, PChar(inttostr(0)))
  else if val > MAX_FADING
    then SetWindowText(Edit, PChar(inttostr(MAX_FADING)));

  SetPageDirty;
end;

procedure TPropPageSettings.ChangeEdit(Edit: THandle; Value: Integer);
var
  val: Integer;
  buf: array[0..255] of Char;
begin
  if GetWindowText(Edit, buf, 256) = 0
    then Exit;
  val := strtoint(buf);
  Value := Value * -1;

  if val < 0 then
  begin
    SetWindowText(Edit, PChar(inttostr(0)));
    SetPageDirty;
    Exit;
  end;

  if val > MAX_FADING then
  begin
    SetWindowText(Edit, PChar(inttostr(MAX_FADING)));
    SetPageDirty;
    Exit;
  end;

  if ((val = 0) and (Value < 0)) or
     ((val = MAX_FADING) and (Value > 0))
    then Exit;

  inc(val, value);
  SetWindowText(Edit, PChar(inttostr(val)));
  SetPageDirty;
end;

procedure TPropPageSettings.RefreshUI;
var
  i: Integer;
  g: TGuid;
  f: Integer;
  b: Boolean;
begin
  for i := 0 to FDeviceList.Count -1
    do SendMessage(FComboDriver, CB_ADDSTRING, 0, LPARAM(PChar(String(PDSDevice(FDeviceList[i]).Name))));

  SetWindowText(FEditSeek, PChar(String('0')));
  SetWindowText(FEditStart, PChar(String('0')));

  if Assigned(FIntf) then
  begin
    FIntf.GetDevice(g);
    for i := 0 to  FDeviceList.Count -1 do
    begin
      if IsEqualGUID(g, PDSDevice(FDeviceList[i]).Guid) then
      begin
        SendMessage(FComboDriver, CB_SETCURSEL, i, 0);
        break;
      end;
    end;
    FIntf.GetFadingSeekStop(f);
    SetWindowText(FEditSeek, PChar(inttostr(f)));
    FIntf.GetFadingCross(f);
    SetWindowText(FEditStart, PChar(inttostr(f)));
    FIntf.GetAllowVideo(b);
    SendMessage(FCheckAllowVideo, BM_SETCHECK, WPARAM(b), 0);
  end;
end;

function TPropPageSettings.OnApplyChanges: HRESULT;
var
  r: Integer;
  buf: array[0..255] of Char;
begin
  Result := S_OK;

  if not Assigned(FIntf)
    then Exit;

  r := SendMessage(FComboDriver, CB_GETCURSEL, 0, 0);

  if (r <> CB_ERR) and (r < FDeviceList.Count)
    then FIntf.SetDevice(PDSDevice(FDeviceList[r]).Guid);

  if GetWindowText(FEditSeek, buf, 256) > 0
    then FIntf.SetFadingSeekStop(strtoint(buf));

  if GetWindowText(FEditStart, buf, 256) > 0
    then FIntf.SetFadingCross(strtoint(buf));

  FIntf.SetAllowVideo(SendMessage(FCheckAllowVideo, BM_GETCHECK, 0, 0) = BST_CHECKED);
end;

constructor TPropPageAbout.CreateFromFactory(Factory: TBCClassFactory; const Controller: IInterface);
begin
  Create(Factory.Name, Controller, 102, 'About');
end;

initialization

  TBCClassFactory.CreatePropertyPage(TPropPageSettings, CLSID_PropPageSettings);
  TBCClassFactory.CreatePropertyPage(TPropPageAbout, CLSID_PropAbout);

end.
